'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.getOutputFilePath = getOutputFilePath;
exports.getFilePath = getFilePath;
exports.getWebpackConfig = getWebpackConfig;
exports.getScopeId = getScopeId;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _hashSum = require('hash-sum');

var _hashSum2 = _interopRequireDefault(_hashSum);

var _config = require('./config');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// require("@babel/register");


function getOutputFilePath(filePath) {
    let relativePath = _path2.default.relative(_config.baseDir, filePath);
    let outputFilePath = _path2.default.resolve(_config.outputPath, relativePath);
    let parsedPath = _path2.default.parse(outputFilePath);
    return {
        outputFileDir: parsedPath.dir,
        outputFilePath
    };
}

function getFilePath(importPath) {
    // 有限判断php，有php的文件会覆盖js转换的实现
    let ext = ['', '.php', '.js', '.vue', '.jsx', '.es6', '/index.php', '/index.js', '/index.vue', '/index.jsx', '/index.es6'];
    for (var i = 0; i < ext.length; i++) {
        let p = importPath + ext[i];
        if (_fs2.default.existsSync(p)) {
            let stats = _fs2.default.statSync(p);
            if (stats.isFile()) {
                return {
                    filePath: p,
                    // 当前名称文件没找到，找到了index文件
                    filePathWithOutExt: i > 5 ? importPath + '/index' : importPath
                };
            }
        }
    }
    return;
}

let webpackConfig;
function getWebpackConfig() {
    if (webpackConfig) {
        return webpackConfig;
    }
    if (_fs2.default.existsSync(_config.webpackConfigPath)) {
        webpackConfig = require(_config.webpackConfigPath);
        return webpackConfig;
    }
    return;
}

function getScopeId(filePath, content = '') {
    let production = process.env.NODE_ENV === 'production';
    // 抄自vue-loader/lib/loader.js#41
    // 做人呢，最重要的就是诚实，抄的就是抄的，不然跟咸鱼还有什么区别
    // 其实大家都觉得看我写的注释能乐一天，笑一笑十年少
    const shortFilePath = _path2.default.relative(_config.baseDir, filePath).replace(/^(\.\.[\\\/])+/, '');
    let moduleId = 'data-v-' + (0, _hashSum2.default)(production ? shortFilePath + '\n' + content : shortFilePath);
    return moduleId;
}